/*
  Copyright (C) 2011 Craciun Dan <craciun.dan@tuxarena.com>

  This file is part of NeonView.

  NeonView is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.

  NeonView is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with NeonView.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "header.h"

void put_error (int status) {
	if (status == 11) {
		fprintf(stderr, "ERROR: Could not allocate memory.\n");
	} else {
		fprintf(stderr, "ERROR: A critical error occured. Program aborted.\n");
	}
	exit(status);
}

int main (int argc, char **argv) {
	TM *tm = NULL;

	/* memory allocation */
	tm = malloc(sizeof(TM));
	if (!tm) { put_error(11); }
	tm->tfd = malloc(sizeof(TFD));
	if (!tm->tfd) { put_error(11); }
	tm->tfi = malloc(sizeof(TFI));
	if (!tm->tfi) { put_error(11); }
	tm->tcfg = malloc(sizeof(TCFG));
	if (!tm->tcfg) { put_error(11); }

	check_config(tm);
	//	read_config(tm);

	tm->fit_images = 0; /* after fit to window button gets pressed, automatically fit to window next/prev images */

	if (tm->tcfg->window_width <= 0 || tm->tcfg->window_width > 1920 * 2) {
		tm->tcfg->window_width = 800;
	}
	if (tm->tcfg->window_height <= 0 || tm->tcfg->window_height > 1440 * 2) {
		tm->tcfg->window_height = 480;
	}


	/* will hold the absolute path from which the application was launched, without the ending slash */
	getcwd(tm->tfd->cwd, sizeof(tm->tfd->cwd));

	gtk_init(&argc, &argv);

	show_widgets(tm);

	if (argc == 1) {
		strcpy(tm->tfd->listd, tm->tfd->cwd);
		list_files(tm);
		sort_files(tm);

		if (tm->tfd->total == 0) {
			gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->next), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), FALSE);
		} else {
			strcpy(tm->tfd->disp, tm->tfd->files[0]);
			char tmp[4097]; int j = 0;
			for (int i = strlen(tm->tfd->disp) - 1; i > strlen(tm->tfd->cwd); i--) {
				tmp[j] = tm->tfd->disp[i];
				j++;
			}
			tmp[j] = '\0';
			rts(tmp);

			strcpy(tm->tfd->basename, tmp);
			tm->tfd->pos = 0;

			put_image(tm);
			if (strcmp(tm->tcfg->remember_fit, "yes") == 0) {
				zoom_fit(tm);
			}
			gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), FALSE);
			if (tm->tfd->total == 1) {
				gtk_widget_set_sensitive(GTK_WIDGET(tm->next), FALSE);
				gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), FALSE);
			}
		}
	}
	if (argc == 2) {
		int found = 0;

		for (int i = 0; i < strlen(argv[1]); i++) {
			if (argv[1][i] == '/') {
				found = 1;
			}
		}
		if (found == 1) {
			char dirname[4097], basename[4097];
			int save = -1;
			int i = strlen(argv[1]) - 1;
			int j = 0;

			while (save == -1) {
				if (argv[1][i] == '/') {
					save = i; // last position where / was found
				} else {
					basename[j] = argv[1][i];
					j++;
				}
				i--;
			}

			basename[j] = '\0';
			rts(basename);
			strcpy(tm->tfd->basename, basename);

			for (int i = 0; i <= save; i++) {
				dirname[i] = argv[1][i];
			}
			dirname[i+1] = '\0';

			char tmp[4097];

			if (argv[1][0] != '/') {
				strcpy(tmp, tm->tfd->cwd);
				strcat(tmp, "/");
				strcat(tmp, dirname);
				strcpy(tm->tfd->disp, argv[1]);
				strcpy(tm->tfd->listd, tmp);
			}

			if (argv[1][0] == '/') {
				strcpy(tm->tfd->disp, argv[1]);
				strcpy(tm->tfd->listd, dirname);
			}

			list_files(tm);
			sort_files(tm);

			strcpy(tm->tfd->basename, basename);
			strcpy(tm->tfd->path, tm->tfd->listd);

			put_image(tm);

			/*
			  chdir(dirname);
			  getcwd(tm->tfd->cwd, sizeof(tm->tfd->cwd));
			*/

			for (i = 0; i < tm->tfd->total; i++) {
				if (strstr(tm->tfd->files[i], tm->tfd->disp) != NULL) {
					tm->tfd->pos = i;
				}
			}
			strcpy(tm->tfd->fullpath, tm->tfd->files[i-1]);

			if (tm->tfd->total == 0 || tm->tfd->total == 1) {
				gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), FALSE);
				gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), FALSE);
				gtk_widget_set_sensitive(GTK_WIDGET(tm->next), FALSE);
				gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), FALSE);
			}
			if (tm->tfd->total > 1) {
				if (tm->tfd->pos == 0) {
					gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), FALSE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), FALSE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->next), TRUE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), TRUE);
				}
				if (tm->tfd->pos == tm->tfd->total - 1) {
					gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), TRUE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), TRUE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->next), FALSE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), FALSE);
				}
			}
		} else if (found == 0) {
			strcpy(tm->tfd->basename, argv[1]);
			strcpy(tm->tfd->disp, argv[1]);
			strcpy(tm->tfd->listd, tm->tfd->cwd);

			list_files(tm);
			sort_files(tm);

			put_image(tm);

			if (tm->tfd->total == 0 || tm->tfd->total == 1) {
				gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), FALSE);
				gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), FALSE);
				gtk_widget_set_sensitive(GTK_WIDGET(tm->next), FALSE);
				gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), FALSE);
			}
			if (tm->tfd->total > 1) {
				if (tm->tfd->pos == 0) {
					gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), FALSE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), FALSE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->next), TRUE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), TRUE);
				}
				if (tm->tfd->pos == tm->tfd->total - 1) {
					gtk_widget_set_sensitive(GTK_WIDGET(tm->prev), TRUE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_prev), TRUE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->next), FALSE);
					gtk_widget_set_sensitive(GTK_WIDGET(tm->menu_next), FALSE);
				}
			}
		}
	}

	gtk_main();

	return 0;
}
